@@ -16,30 +16,17 @@ from trytond.pyson import If, Eval, Bool, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
+from trytond.ir.attachment import AttachmentCopyMixin
+from trytond.ir.note import NoteCopyMixin
 from trytond.modules.account.tax import TaxableMixin
 from trytond.modules.account_product.exceptions import AccountError
-from trytond.modules.product import price_digits
+from trytond.modules.company.model import (
+    employee_field, set_employee, reset_employee)
+from trytond.modules.product import price_digits, round_price
+
 from .exceptions import (
     SaleValidationError, SaleQuotationError, PartyLocationError)
 
-__all__ = ['Sale', 'SaleIgnoredInvoice', 'SaleRecreatedInvoice',
-    'SaleLine', 'SaleLineTax', 'SaleLineIgnoredMove',
-    'SaleLineRecreatedMove', 'SaleReport', 'OpenCustomer',
-    'HandleShipmentExceptionAsk', 'HandleShipmentException',
-    'HandleInvoiceExceptionAsk', 'HandleInvoiceException',
-    'ReturnSaleStart', 'ReturnSale', 'ModifyHeader',
-    'get_shipments_returns', 'search_shipments_returns']
-
-_ZERO = Decimal(0)
-STATES = [
-    ('draft', 'Draft'),
-    ('quotation', 'Quotation'),
-    ('confirmed', 'Confirmed'),
-    ('processing', 'Processing'),
-    ('done', 'Done'),
-    ('cancel', 'Canceled'),
-    ]
-
 
 def get_shipments_returns(model_name):
     "Computes the returns or shipments"
@@ -71,7 +58,9 @@ def search_shipments_returns(model_name):
     return classmethod(method)
 
 
-class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
+class Sale(
+        Workflow, ModelSQL, ModelView, TaxableMixin,
+        AttachmentCopyMixin, NoteCopyMixin):
     'Sale'
     __name__ = 'sale.sale'
     _rec_name = 'number'
@@ -91,7 +80,6 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             'readonly': Eval('state') != 'draft',
             },
         depends=['state'])
-    state = fields.Selection(STATES, 'State', readonly=True, required=True)
     sale_date = fields.Date('Sale Date',
         states={
             'readonly': ~Eval('state').in_(['draft', 'quotation']),
@@ -112,11 +100,27 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         depends=['state'])
     party_lang = fields.Function(fields.Char('Party Language'),
         'on_change_with_party_lang')
+    contact = fields.Many2One(
+        'party.contact_mechanism', "Contact",
+        search_context={
+            'related_party': Eval('party'),
+            },
+        depends=['party'])
+    invoice_party = fields.Many2One('party.party', "Invoice Party",
+        states={
+            'readonly': ((Eval('state') != 'draft')
+                | Eval('lines', [0])),
+            },
+        depends=['state'])
     invoice_address = fields.Many2One('party.address', 'Invoice Address',
-        domain=[('party', '=', Eval('party'))], states={
+        domain=[
+            ('party', '=', If(Bool(Eval('invoice_party')),
+                    Eval('invoice_party'), Eval('party'))),
+            ],
+        states={
             'readonly': Eval('state') != 'draft',
             'required': ~Eval('state').in_(['draft', 'quotation', 'cancel']),
-            }, depends=['state', 'party'])
+            }, depends=['party', 'invoice_party', 'state'])
     shipment_party = fields.Many2One('party.party', 'Shipment Party',
         states={
             'readonly': (Eval('state') != 'draft'),
@@ -224,6 +228,21 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             },
         depends=['state'])
 
+    quoted_by = employee_field(
+        "Quoted By",
+        states=['quotation', 'confirmed', 'processing', 'done', 'cancel'])
+    confirmed_by = employee_field(
+        "Confirmed By",
+        states=['confirmed', 'processing', 'done', 'cancel'])
+    state = fields.Selection([
+            ('draft', "Draft"),
+            ('quotation', "Quotation"),
+            ('confirmed', "Confirmed"),
+            ('processing', "Processing"),
+            ('done', "Done"),
+            ('cancel', "Canceled"),
+            ], "State", readonly=True, required=True)
+
     @classmethod
     def __setup__(cls):
         super(Sale, cls).__setup__()
@@ -365,19 +384,29 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
     def default_shipment_state():
         return 'none'
 
-    @fields.depends('party', 'shipment_party', 'payment_term')
+    @fields.depends('party', 'invoice_party', 'shipment_party', 'payment_term')
     def on_change_party(self):
-        self.invoice_address = None
+        if not self.invoice_party:
+            self.invoice_address = None
         if not self.shipment_party:
             self.shipment_address = None
         self.payment_term = self.default_payment_term()
         if self.party:
-            self.invoice_address = self.party.address_get(type='invoice')
+            if not self.invoice_party:
+                self.invoice_address = self.party.address_get(type='invoice')
             if not self.shipment_party:
                 self.shipment_address = self.party.address_get(type='delivery')
             if self.party.customer_payment_term:
                 self.payment_term = self.party.customer_payment_term
 
+    @fields.depends('party', 'invoice_party')
+    def on_change_invoice_party(self):
+        if self.invoice_party:
+            self.invoice_address = self.invoice_party.address_get(
+                type='invoice')
+        elif self.party:
+            self.invoice_address = self.party.address_get(type='invoice')
+
     @fields.depends('party', 'shipment_party')
     def on_change_shipment_party(self):
         if self.shipment_party:
@@ -392,6 +421,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             return self.currency.digits
         return 2
 
+    @fields.depends('party')
     def _get_tax_context(self):
         res = {}
         if self.party and self.party.lang:
@@ -405,7 +435,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             return self.party.lang.code
         return Config.get_language()
 
-    @fields.depends('lines', 'currency', 'party')
+    @fields.depends('lines', 'currency', methods=['_get_taxes'])
     def on_change_lines(self):
         self.untaxed_amount = Decimal('0.0')
         self.tax_amount = Decimal('0.0')
@@ -472,7 +502,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             else:
                 untaxed_amount[sale.id] = sum(
                     (line.amount for line in sale.lines
-                        if line.type == 'line'), _ZERO)
+                        if line.type == 'line'), Decimal(0))
                 if compute_taxes:
                     tax_amount[sale.id] = sale.get_tax_amount()
                     total_amount[sale.id] = (
@@ -652,6 +682,14 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
                     ])
         return attributes
 
+    @classmethod
+    def get_resources_to_copy(cls, name):
+        return {
+            'stock.shipment.out',
+            'stock.shipment.out.return',
+            'account.invoice',
+            }
+
     @classmethod
     def copy(cls, sales, default=None):
         if default is None:
@@ -725,13 +763,14 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         'Return invoice'
         pool = Pool()
         Invoice = pool.get('account.invoice')
+        party = self.invoice_party or self.party
         invoice = Invoice(
             company=self.company,
             type='out',
-            party=self.party,
+            party=party,
             invoice_address=self.invoice_address,
             currency=self.currency,
-            account=self.party.account_receivable_used,
+            account=party.account_receivable_used,
             )
         invoice.on_change_type()
         invoice.payment_term = self.payment_term
@@ -758,6 +797,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         invoice.save()
 
         Invoice.update_taxes([invoice])
+        self.copy_resources_to(invoice)
         return invoice
 
     def _group_shipment_key(self, moves, move):
@@ -783,10 +823,9 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
 
     def _get_shipment_sale(self, Shipment, key):
         values = {
-            'customer': (self.shipment_party.id if self.shipment_party
-                else self.party.id),
-            'delivery_address': self.shipment_address.id,
-            'company': self.company.id,
+            'customer': self.shipment_party or self.party,
+            'delivery_address': self.shipment_address,
+            'company': self.company,
             }
         values.update(dict(key))
         return Shipment(**values)
@@ -822,6 +861,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             shipment.moves = (list(getattr(shipment, 'moves', []))
                 + [x[1] for x in grouped_moves])
             shipment.save()
+            self.copy_resources_to(shipment)
             shipments.append(shipment)
         if shipment_type == 'out':
             Shipment.wait(shipments)
@@ -855,12 +895,14 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
+    @reset_employee('quoted_by', 'confirmed_by')
     def draft(cls, sales):
         pass
 
     @classmethod
     @ModelView.button
     @Workflow.transition('quotation')
+    @set_employee('quoted_by')
     def quote(cls, sales):
         for sale in sales:
             sale.check_for_quotation()
@@ -869,6 +911,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
     @classmethod
     @ModelView.button
     @Workflow.transition('confirmed')
+    @set_employee('confirmed_by')
     def confirm(cls, sales):
         pool = Pool()
         Configuration = pool.get('sale.configuration')
@@ -903,6 +946,9 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
     @classmethod
     @ModelView.button
     def process(cls, sales):
+        pool = Pool()
+        Line = pool.get('sale.line')
+        lines = []
         done = []
         process = []
         cls.lock(sales)
@@ -914,6 +960,11 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             sale.create_shipment('out')
             sale.create_shipment('return')
             sale.set_shipment_state()
+
+            for line in sale.lines:
+                line.set_actual_quantity()
+                lines.append(line)
+
             if sale.is_done():
                 if sale.state != 'done':
                     if sale.state == 'confirmed':
@@ -921,6 +972,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
                     done.append(sale)
             elif sale.state != 'processing':
                 process.append(sale)
+        Line.save(lines)
         if process:
             cls.proceed(process)
         if done:
@@ -956,7 +1008,7 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
     'Sale Line'
     __name__ = 'sale.line'
     sale = fields.Many2One('sale.sale', 'Sale', ondelete='CASCADE',
-        select=True,
+        select=True, required=True,
         states={
             'readonly': ((Eval('sale_state') != 'draft')
                 & Bool(Eval('sale'))),
@@ -980,6 +1032,12 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             'readonly': Eval('sale_state') != 'draft',
             },
         depends=['type', 'unit_digits', 'sale_state'])
+    actual_quantity = fields.Float(
+        "Actual Quantity", digits=(16, Eval('unit_digits', 2)), readonly=True,
+        states={
+            'invisible': Eval('type') != 'line',
+            },
+        depends=['unit_digits', 'type'])
     unit = fields.Many2One('product.uom', 'Unit', ondelete='RESTRICT',
             states={
                 'required': Bool(Eval('product')),
@@ -1011,15 +1069,14 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             'stock_date_end': Eval('_parent_sale', {}).get('sale_date'),
             'stock_skip_warehouse': True,
             # From _get_context_sale_price
-            'company': Eval(
-                '_parent_sale', {}).get('company', None),
+            'company': Eval('company', None),
             'currency': Eval('_parent_sale', {}).get('currency'),
             'customer': Eval('_parent_sale', {}).get('party'),
             'sale_date': Eval('_parent_sale', {}).get('sale_date'),
             'uom': Eval('unit'),
             'taxes': Eval('taxes', []),
             'quantity': Eval('quantity'),
-            }, depends=['type', 'sale_state'])
+            }, depends=['type', 'sale_state', 'company'])
     product_uom_category = fields.Function(
         fields.Many2One('product.uom.category', 'Product Uom Category'),
         'on_change_with_product_uom_category')
@@ -1075,8 +1132,12 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                 },
             depends=['type']),
         'on_change_with_shipping_date')
-    sale_state = fields.Function(fields.Selection(STATES, 'Sale State'),
+    sale_state = fields.Function(
+        fields.Selection('get_sale_states', "Sale State"),
         'on_change_with_sale_state')
+    company = fields.Function(
+        fields.Many2One('company.company', "Company"),
+        'on_change_with_company')
 
     @classmethod
     def __register__(cls, module_name):
@@ -1140,7 +1201,7 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
 
     @fields.depends(
         'sale', '_parent_sale.currency', '_parent_sale.party',
-        '_parent_sale.sale_date', '_parent_sale.company',
+        '_parent_sale.sale_date', 'company',
         'unit', 'product', 'taxes')
     def _get_context_sale_price(self):
         context = {}
@@ -1150,8 +1211,8 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             if self.sale.party:
                 context['customer'] = self.sale.party.id
             context['sale_date'] = self.sale.sale_date
-            if self.sale.company:
-                context['company'] = self.sale.company.id
+        if self.company:
+            context['company'] = self.company.id
         if self.unit:
             context['uom'] = self.unit.id
         elif self.product:
@@ -1159,7 +1220,8 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
         context['taxes'] = [t.id for t in self.taxes or []]
         return context
 
-    @fields.depends('product', 'unit', 'sale', '_parent_sale.party',
+    @fields.depends('product', 'unit', 'sale',
+        '_parent_sale.party', '_parent_sale.invoice_party',
         methods=['compute_taxes', 'compute_unit_price',
             'on_change_with_amount'])
     def on_change_product(self):
@@ -1167,8 +1229,8 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             return
 
         party = None
-        if self.sale and self.sale.party:
-            party = self.sale.party
+        if self.sale:
+            party = self.sale.invoice_party or self.sale.party
 
         # Set taxes before unit_price to have taxes in context of sale price
         self.taxes = self.compute_taxes(party)
@@ -1213,8 +1275,7 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             unit_price = Product.get_sale_price([self.product],
                 self.quantity or 0)[self.product.id]
             if unit_price:
-                unit_price = unit_price.quantize(
-                    Decimal(1) / 10 ** self.__class__.unit_price.digits[1])
+                unit_price = round_price(unit_price)
             return unit_price
 
     @fields.depends('product')
@@ -1300,11 +1361,22 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                 shipping_date = max(shipping_date, Date.today())
             return shipping_date
 
+    @classmethod
+    def get_sale_states(cls):
+        pool = Pool()
+        Sale = pool.get('sale.sale')
+        return Sale.fields_get(['state'])['state']['selection']
+
     @fields.depends('sale', '_parent_sale.state')
     def on_change_with_sale_state(self, name=None):
         if self.sale:
             return self.sale.state
 
+    @fields.depends('sale', '_parent_sale.company')
+    def on_change_with_company(self, name=None):
+        if self.sale and self.sale.company:
+            return self.sale.company.id
+
     def get_invoice_line(self):
         'Return a list of invoice lines for sale line'
         pool = Pool()
@@ -1342,7 +1414,7 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             invoice_line.account = self.product.account_revenue_used
             if not invoice_line.account:
                 raise AccountError(
-                    gettext('sale.msg_sale_product_missing_account_expense',
+                    gettext('sale.msg_sale_product_missing_account_revenue',
                         sale=self.sale.rec_name,
                         product=self.product.rec_name))
         else:
@@ -1350,7 +1422,7 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                 'default_category_account_revenue')
             if not invoice_line.account:
                 raise AccountError(
-                    gettext('sale.msg_sale_missing_account_expense',
+                    gettext('sale.msg_sale_missing_account_revenue',
                         sale=self.sale.rec_name))
         invoice_line.stock_moves = self._get_invoice_line_moves()
         return [invoice_line]
@@ -1371,9 +1443,18 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
         elif self.sale.invoice_method == 'shipment':
             quantity = 0.0
             for move in self.moves:
-                if move.state == 'done':
-                    quantity += Uom.compute_qty(move.uom, move.quantity,
-                        self.unit)
+                if move.state != 'done':
+                    continue
+                qty = Uom.compute_qty(move.uom, move.quantity, self.unit)
+                # Test only against to_location
+                # as it is what matters for sale
+                dest_type = self.to_location.type
+                if (move.to_location.type == dest_type
+                        and move.from_location.type != dest_type):
+                    quantity += qty
+                elif (move.from_location.type == dest_type
+                        and move.to_location.type != dest_type):
+                    quantity -= qty
             if self.quantity < 0:
                 quantity *= -1
             return quantity
@@ -1503,9 +1584,36 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                         invoice_lines.append(invoice_line)
         return invoice_lines
 
+    def set_actual_quantity(self):
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        if self.type != 'line':
+            return
+        moved_quantity = 0
+        for move in self.moves:
+            if move.state != 'cancel':
+                moved_quantity += Uom.compute_qty(
+                    move.uom, move.quantity, self.unit)
+        if self.quantity < 0:
+            moved_quantity *= -1
+        invoiced_quantity = 0
+        for invoice_line in self.invoice_lines:
+            if (not invoice_line.invoice
+                    or invoice_line.invoice.state != 'cancel'):
+                invoiced_quantity += Uom.compute_qty(
+                    invoice_line.unit, invoice_line.quantity, self.unit)
+        self.actual_quantity = max(moved_quantity, invoiced_quantity, key=abs)
+
     def get_rec_name(self, name):
+        pool = Pool()
+        Lang = pool.get('ir.lang')
         if self.product:
-            return '%s @ %s' % (self.product.rec_name, self.sale.rec_name)
+            lang = Lang.get()
+            return (lang.format(
+                    '%.*f', (self.unit.digits, self.quantity or 0))
+                + '%s %s @ %s' % (
+                    self.unit.symbol, self.product.rec_name,
+                    self.sale.rec_name))
         else:
             return self.sale.rec_name
 
