@@ -9,9 +9,9 @@ except ImportError:
     pygal = None
 from dateutil.relativedelta import relativedelta
 from sql import Null, Literal, Column
-from sql.aggregate import Sum, Min, Count
-from sql.conditionals import Case
-from sql.functions import CurrentTimestamp, DateTrunc
+from sql.aggregate import Sum, Max, Min, Count
+from sql.conditionals import Coalesce
+from sql.functions import CurrentTimestamp, DateTrunc, Power, Ceil, Log
 
 from trytond.pool import Pool
 from trytond.model import ModelSQL, ModelView, UnionMixin, fields
@@ -19,14 +19,7 @@ from trytond.tools import grouped_slice, reduce_ids
 from trytond.transaction import Transaction
 from trytond.pyson import Eval, If
 from trytond.wizard import Wizard, StateTransition, StateAction
-
-__all__ = ['Context',
-    'Customer', 'CustomerTimeseries',
-    'Product', 'ProductTimeseries',
-    'Category', 'CategoryTimeseries', 'CategoryTree',
-    'Country', 'CountryTimeseries',
-    'Subdivision', 'SubdivisionTimeseries',
-    'Region', 'OpenRegion']
+from trytond.i18n import lazy_gettext
 
 
 def pairwise(iterable):
@@ -37,17 +30,23 @@ def pairwise(iterable):
 
 class Abstract(ModelSQL):
 
-    company = fields.Many2One('company.company', "Company")
-    number = fields.Integer("#", help="Number of sales")
+    company = fields.Many2One(
+        'company.company', lazy_gettext("sale.msg_sale_reporting_company"))
+    number = fields.Integer(lazy_gettext("sale.msg_sale_reporting_number"),
+        help=lazy_gettext("sale.msg_sale_reporting_number_help"))
     revenue = fields.Numeric(
-        "Revenue", digits=(16, Eval('currency_digits', 2)),
+        lazy_gettext("sale.msg_sale_reporting_revenue"),
+        digits=(16, Eval('currency_digits', 2)),
         depends=['currency_digits'])
     revenue_trend = fields.Function(
-        fields.Char("Revenue Trend"), 'get_trend')
+        fields.Char(lazy_gettext("sale.msg_sale_reporting_revenue_trend")),
+        'get_trend')
     time_series = None
 
     currency_digits = fields.Function(
-        fields.Integer("Currency Digits"), 'get_currency_digits')
+        fields.Integer(
+            lazy_gettext("sale.msg_sale_reporting_currency_digits")),
+        'get_currency_digits')
 
     @classmethod
     def table_query(cls):
@@ -97,8 +96,9 @@ class Abstract(ModelSQL):
         currency_company = tables['currency_company']
         currency_sale = tables['currency_sale']
 
+        quantity = Coalesce(line.actual_quantity, line.quantity)
         revenue = cls.revenue.sql_cast(
-            Sum(line.quantity * line.unit_price
+            Sum(quantity * line.unit_price
                 * currency_company.rate / currency_sale.rate))
         return [
             cls._column_id(tables).as_('id'),
@@ -378,13 +378,16 @@ class CategoryMixin(object):
 
     @classmethod
     def _column_id(cls, tables):
+        pool = Pool()
+        Category = pool.get('product.category')
+        category = Category.__table__()
         line = tables['line']
         template_category = tables['line.product.template_category']
-        # Pairing function from http://szudzik.com/ElegantPairing.pdf
-        return Min(Case(
-                (line.id < template_category.id,
-                    (template_category.id * template_category.id) + line.id),
-                else_=(line.id * line.id) + line.id + template_category.id))
+        # Get a stable number of category over time
+        # by using number one order bigger.
+        nb_category = category.select(
+            Power(10, (Ceil(Log(Max(category.id))) + Literal(1))))
+        return Min(line.id * nb_category + template_category.id)
 
     @classmethod
     def _group_by(cls, tables):
